<?php
/**
 * @package     CMShortURL
 * @subpackage  com_cmshorturl
 * @copyright   Copyright (C) 2015 CMExtension Team http://www.cmext.vn/
 * @license     GNU General Public License version 2 or later
 */

defined('_JEXEC') or die;

/**
 * Model class for url list.
 *
 * @package     CMShortURL
 * @subpackage  com_cmshorturl
 * @since       1.0.0
 */
class CMShortURLModelUrls extends JModelList
{
	/**
	 * Constructor.
	 *
	 * @param   array  $config  An optional associative array of configuration settings.
	 *
	 * @since   1.0.0
	 */
	public function __construct($config = array())
	{
		if (empty($config['filter_fields']))
		{
			$config['filter_fields'] = array(
				'id', 'a.id',
				'alias', 'a.alias',
				'campaign_id', 'a.campaign_id',
				'source_id', 'a.source_id',
			);
		}

		parent::__construct($config);
	}

	/**
	 * Method to auto-populate the model state.
	 *
	 * Note. Calling getState in this method will result in recursion.
	 *
	 * @param   string  $ordering   An optional ordering fiela.
	 * @param   string  $direction  An optional direction (asc|desc).
	 *
	 * @return  void
	 *
	 * @since   1.0.0
	 */
	protected function populateState($ordering = null, $direction = null)
	{
		// Load the filter state.
		$search = $this->getUserStateFromRequest($this->context . '.filter.search', 'filter_search');
		$this->setState('filter.search', $search);

		$campaign = $this->getUserStateFromRequest($this->context . '.filter.campaign', 'filter_campaign_id');
		$this->setState('filter.campaign', $campaign);

		$source = $this->getUserStateFromRequest($this->context . '.filter.source', 'filter_source_id');
		$this->setState('filter.source', $source);

		$content = $this->getUserStateFromRequest($this->context . '.filter.content', 'filter_content_id');
		$this->setState('filter.content', $content);

		// Load the parameters.
		$params = JComponentHelper::getParams('com_cmshorturl');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.id', 'desc');
	}

	/**
	 * Method to get a store id based on model configuration state.
	 *
	 * This is necessary because the model is used by the component and
	 * different modules that might need different sets of data or different
	 * ordering requirements.
	 *
	 * @param   string  $id  A prefix for the store ia.
	 *
	 * @return  string  A store id.
	 *
	 * @since   1.0.0
	 */
	protected function getStoreId($id = '')
	{
		// Compile the store id.
		$id .= ':' . $this->getState('filter.search');
		$id .= ':' . $this->getState('filter.campaign');
		$id .= ':' . $this->getState('filter.source');
		$id .= ':' . $this->getState('filter.content');

		return parent::getStoreId($id);
	}

	/**
	 * Build an SQL query to load the list data.
	 *
	 * @return  JDatabaseQuery
	 *
	 * @since   1.0.0
	 */
	protected function getListQuery()
	{
		// Create a new query object.
		$db = $this->getDbo();
		$query = $db->getQuery(true);
		$user = JFactory::getUser();

		// Select the required fields from the table.
		$query->select(
				$this->getState(
					'list.select',
					'a.id, a.alias, a.url, a.hits, a.checked_out, a.checked_out_time'
				)
			)
			->from($db->qn('#__cmshorturl_urls') . ' AS a');

		// Join over the users for the checked out user.
		$query->select($db->qn('uc.name') . ' AS editor')
			->join('LEFT', $db->qn('#__users') . ' AS uc ON '
				. $db->qn('uc.id') . ' = ' . $db->qn('a.checked_out')
			);

		$query->select($db->qn('p.utm_campaign') . ' AS utm_campaign')
			->select($db->qn('p.utm_term') . ' AS utm_term')
			->select($db->qn('p.utm_medium') . ' AS utm_medium')
			->join('LEFT', $db->qn('#__cmshorturl_campaigns') . ' AS p ON '
				. $db->qn('p.id') . ' = ' . $db->qn('a.campaign_id')
			);

		$query->select($db->qn('s.utm_source') . ' AS utm_source')
			->join('LEFT', $db->qn('#__cmshorturl_sources') . ' AS s ON '
				. $db->qn('s.id') . ' = ' . $db->qn('a.source_id')
			);

		$query->select($db->qn('c.utm_content') . ' AS utm_content')
			->join('LEFT', $db->qn('#__cmshorturl_contents') . ' AS c ON '
				. $db->qn('c.id') . ' = ' . $db->qn('a.content_id')
			);

		$search = $this->getState('filter.search');

		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0)
			{
				$query->where($db->qn('a.id') . ' = ' . (int) substr($search, 3));
			}
			else
			{
				$search = $db->q('%' . str_replace(' ', '%', $db->escape(trim($search), true) . '%'));
				$query->where(
					'(' . $db->qn('a.alias') . ' LIKE ' . $search . ' OR ' .
						$db->qn('a.url') . ' LIKE ' . $search . ' OR ' .
						$db->qn('utm_campaign') . ' LIKE ' . $search . ' OR ' .
						$db->qn('utm_content') . ' LIKE ' . $search . ' OR ' .
						$db->qn('utm_term') . ' LIKE ' . $search . ' OR ' .
						$db->qn('utm_source') . ' LIKE ' . $search . ' OR ' .
						$db->qn('utm_medium') . ' LIKE ' . $search . ')'
				);
			}
		}

		// Filter by campaign.
		$campaignId = $this->getState('filter.campaign');

		if (is_numeric($campaignId))
		{
			$query->where($db->qn('a.campaign_id') . ' = ' . $db->q((int) $campaignId));
		}

		// Filter by source.
		$sourceId = $this->getState('filter.source');

		if (is_numeric($sourceId))
		{
			$query->where($db->qn('a.source_id') . ' = ' . $db->q((int) $sourceId));
		}

		// Filter by content.
		$contentId = $this->getState('filter.content');

		if (is_numeric($contentId))
		{
			$query->where($db->qn('a.content_id') . ' = ' . $db->q((int) $contentId));
		}

		// Add the list ordering clause.
		$orderCol = $this->state->get('list.ordering', 'a.id');
		$orderDirn = $this->state->get('list.direction', 'desc');

		$query->order($db->escape($orderCol . ' ' . $orderDirn));

		return $query;
	}

	/**
	 * Get the filter form
	 *
	 * @param   array    $data      data
	 * @param   boolean  $loadData  load current data
	 *
	 * @return  JForm/false  the JForm object or false
	 *
	 * @since   1.0.0
	 */
	public function getFilterForm($data = array(), $loadData = true)
	{
		$form = null;

		// Try to locate the filter form automatically. Example: ContentModelArticles => "filter_articles"
		if (empty($this->filterFormName))
		{
			$classNameParts = explode('Model', get_called_class());

			if (count($classNameParts) == 2)
			{
				$this->filterFormName = 'filter_' . strtolower($classNameParts[1]);
			}
		}

		if (!empty($this->filterFormName))
		{
			// Get the form.
			$form = $this->loadForm($this->context . '.filter', $this->filterFormName, array('control' => '', 'load_data' => $loadData));
		}

		if (!$this->state->get('params')->get('utm_params', 0))
		{
			$form->removeField('campaign_id', 'filter');
			$form->removeField('source_id', 'filter');
			$form->removeField('content_id', 'filter');
		}

		return $form;
	}
}
